

初始的系统比较简单，还可以添加很多用户常用的功能，我们新增了date、uname、pwd、touch的用户功能，便于用户更方便地了解系统

# 一、date

**功能:**提供当前系统时间，返回日期和时间数据到用户空间。由于是移植开发板上，所以使用的是 RTC 时间，而在qemu中没有RTC，可以基于 CLINT `mtime` 寄存器进行时钟获取。

## 原理分析

`date` 系统调用的核心是获取当前系统的时间，并将其转换为可读的日期格式（年月日时分秒）。在 RISC-V 平台上，时间的来源是 **CLINT (Core Local Interruptor)** 中的 `mtime` 寄存器。

### **CLINT 和 `mtime`**

**1. 什么是 CLINT**

CLINT 是 RISC-V 平台上的 **Core Local Interruptor**，用于管理内核本地的中断，比如计时器中断和软件中断。

它包含以下两个核心功能：

**Timer**：提供全局时钟和计时器功能，通过 `mtime` 和 `mtimecmp` 寄存器实现。

**Software Interrupts**：允许处理器核心间通过写入特定寄存器发起中断。



**2. `mtime` 寄存器**

`mtime` 是 CLINT 提供的全局计时器寄存器，用于记录系统启动后经过的时钟周期数。

其地址在 CLINT 的内存映射区域中，具体为：

- **物理地址**：`0x0200_BFF8`。
- **虚拟地址**（加上 `VIRT_OFFSET`）：`CLINT_MTIME = CLINT + VIRT_OFFSET + 0xBFF8`。

- 通过读取 `mtime` 的值，可以获得当前的时钟周期数。

    

**3. 时间单位转换**

CLINT 的 `mtime` 记录的是以系统时钟（`CPU Clock`）为单位的周期数，因此需要转换为秒。

转换公式：

```
seconds = mtime / TICKS_PER_SECOND;
```

`TICKS_PER_SECOND`：系统时钟每秒的周期数（通常取决于硬件频率，如 QEMU 模拟时为 10,000,000）。

## **实现流程**

用户执行date程序，内部会通过sys_date系统调用返回数据，然后进行打印，系统调用流程如下

1. 获取系统时钟：使用 `CLINT_MTIME` 寄存器读取当前系统时钟计数（以 CPU 时钟周期计数）。将时钟周期转换为秒。
2. 转换为日期格式：使用 `convert_to_date` 函数，将时间秒数转换为标准的日期结构（`struct rtcdate`）。日期结构包括：年、月、日、小时、分钟、秒。
3. 参数解析：从用户空间解析参数，获取用户提供的缓冲区地址 `user_dst`。
4. 将数据写回用户空间：使用 `copyout` 将日期结构写入用户空间指定的缓冲区地址。
5. 返回结果：成功返回 `0`，失败返回 `-1`。

关键数据结构

```
struct rtcdate {
    int year;
    int month;
    int day;
    int hour;
    int minute;
    int second;
};
```

**代码分析**

时间获取：

TICKS_PER_SECOND是QEMU平台默认的时钟频率，可根据实际调整

```
uint64 time_in_seconds = *(volatile uint64 *)CLINT_MTIME / TICKS_PER_SECOND;
convert_to_date(time_in_seconds, &r);	// 将秒转换为日期格式
```

用户空间数据写入：

```
if (copyout(curproc->pagetable, (uint64)user_dst, (char *)&r, sizeof(r)) < 0) {
    return -1;  // 如果写入用户空间失败，返回错误
}
```

用户程序中可以根据需要修改打印的格式，比如只打印系统运行了的时间，以时分秒/日期+时分秒输出，日期可以自定义开始时间。如果是在搭载RTC模块的开发板上，可以根据实际时间对时。



# 二、uname

**功能:**获取当前操作系统的基本信息，包括系统名称、版本号、机器架构等。将信息返回到用户提供的缓冲区中。

比较容易实现。

## **实现流程**

1. 解析用户参数：从用户空间解析参数，获取用户提供的缓冲区地址 `uname_buf`。
2. 构造系统信息：构造 `struct utsname` 数据结构，填充系统的基本信息：
3. 数据拷贝到用户空间：使用 `copyout` 将填充的 `struct utsname` 数据结构拷贝到用户空间指定的缓冲区地址。
4. 返回结果：成功返回 `0`，失败返回 `-1`。

关键数据结构:

```
struct utsname {
    char sysname[65];   // 系统名称
    char nodename[65];  // 主机名
    char release[65];   // 系统版本
    char version[65];   // 详细版本
    char machine[65];   // 机器架构
};
```

**代码分析:**

构造系统信息：

```
struct utsname info;
safestrcpy(info.machine, UTS_MACHINE, sizeof(info.machine));
```

用户空间数据写入：

```
if (copyout(myproc()->pagetable, (uint64)uname_buf, (char *)&info, sizeof(info)) < 0) {
    return -1;  // 如果写入用户空间失败，返回错误
}
```

UTS_MACHINE可以根据启动环境自动识别qemu或者k210，不同的字段可以根据不同的条件进行选择，实现动态生成。

# 三、touch

功能：实现创建文件功能

## 实现流程：

仿照mkdir的实现，只是在create的时候将T_DIR改为T_FILE

```
uint64
sys_touch(void)
{
  char path[FAT32_MAX_PATH];
  struct dirent *ep;

  if(argstr(0, path, FAT32_MAX_PATH) < 0 || (ep = create(path, T_FILE, 0)) == 0){
    return -1;
  }
  eunlock(ep);
  eput(ep);
  return 0;
}
```

# 四、pwd

功能：输出当前的路径

## 实现流程

```
uint64
sys_pwd(void)
{
  uint64 addr;
  if (argaddr(0, &addr) < 0)
    return -1;

  struct dirent *de = myproc()->cwd;
  char path[FAT32_MAX_PATH];
  char *s;
  int len;

  if (de->parent == NULL) {
    s = "/";
  } else {
    s = path + FAT32_MAX_PATH - 1;
    *s = '\0';
    while (de->parent) {
      len = strlen(de->filename);
      s -= len;
      if (s <= path)          // can't reach root "/"
        return -1;
      strncpy(s, de->filename, len);
      *--s = '/';
      de = de->parent;
    }
  }

  // if (copyout(myproc()->pagetable, addr, s, strlen(s) + 1) < 0)
  if (copyout2(addr, s, strlen(s) + 1) < 0)
    return -1;
  
  return 0;

}
```

1. **获取用户空间地址**：

   调用 `argaddr(0, &addr)` 获取用户空间传入的地址 `addr`，用于存储绝对路径字符串。如果获取失败，返回 -1。

2. **初始化变量**：

   获取当前进程的当前工作目录 `cwd`，存储在 `struct dirent *de` 中。

   定义一个字符数组 `path` 用于存储路径，定义一个字符指针 `s` 和一个整数 `len`。

3. **处理根目录情况**：

   如果当前目录的父目录为 `NULL`，说明当前目录是根目录，直接将 `s` 设置为根目录字符串 `"/"`。

4. **构建路径字符串**：

   如果当前目录不是根目录，从路径数组的末尾开始构建路径字符串。

   使用while循环遍历目录项，直到到达根目录。

   获取当前目录项的文件名长度 `len`。

   将指针 `s` 向前移动 `len` 个位置。

   检查指针 `s` 是否越界，如果越界返回 -1。

   将当前目录项的文件名复制到指针 `s` 所在位置。

   在文件名前添加斜杠 `"/"`。

   将当前目录项更新为其父目录项。

5. **复制路径字符串到用户空间**：

   使用 `copyout2(addr, s, strlen(s) + 1)` 将构建好的路径字符串复制到用户空间地址 `addr`。如果复制失败，返回 -1。

6. **返回成功**：

   如果一切顺利，返回 0 表示成功。