#ifndef _MMAP_H_
#define _MMAP_H_

#include "types.h"

#define PROT_NONE		0
#define PROT_READ		1
#define PROT_WRITE		2
#define PROT_EXEC		4
#define PROT_GROWSDOWN	0x01000000
#define PROT_GROWSUP	0x02000000
#define PROT_ALL (PROT_READ|PROT_WRITE|PROT_EXEC|PROT_GROWSDOWN|PROT_GROWSUP)

#define MAP_FILE		0
#define MAP_SHARED		0x01
#define MAP_PRIVATE		0x02
#define MAP_FIXED		0x10
#define MAP_ANONYMOUS	0x20
#define MAP_FAILED ((void *) -1)

#define MS_ASYNC		1
#define MS_INVALIDATE	2
#define MS_SYNC			4


struct mmap_info {
    int used;              // 标记是否已使用。1 表示该映射已经被使用，0 表示未使用。
    uint64 start;          // 映射的起始地址，表示内存映射的起始虚拟地址。
    uint64 len;            // 映射区域的长度，以字节为单位。
    int prot;              // 内存保护标志，例如 PROT_READ, PROT_WRITE, PROT_EXEC 等。
    int flags;             // 映射标志，用于指定内存映射类型（如 MAP_PRIVATE, MAP_SHARED 等）。
    int fd;                // 文件描述符，表示要映射的文件。如果是匿名映射，通常为 -1。
    off_t offset;          // 映射的文件偏移量，表示从文件的哪个位置开始映射。
};

uint64 do_mmap(uint64 start, uint64 len,int prot,int flags,int fd, off_t offset);
uint64 do_munmap(uint64 start, uint64 len);

#endif
