// init: The initial user-level program

#include "kernel/include/types.h"
#include "kernel/include/stat.h"
#include "kernel/include/file.h"
#include "kernel/include/fat32.h"
#include "kernel/include/fcntl.h"
#include "xv6-user/user.h"

char *argv[] = { "sh", 0 };


//将两个字符串（路径和文件名）拼接到一个缓冲区中的功能，确保不会溢出缓冲区。
void build_path(char *buf, const char *path, const char *filename, int bufsize) {
    int len_path = strlen(path);
    int len_file = strlen(filename);
    int total_len = len_path + len_file + 1; // +1 是为了加上 '/'

    // 检查总长度是否超出缓冲区
    if (total_len >= bufsize) {
        printf("Error: buffer overflow in build_path\n");
        return;
    }

    // 清空缓冲区
    memset(buf, 0, bufsize);

    // 复制路径到 buf
    strcpy(buf, path);

    // 如果路径结尾没有 '/'，添加 '/'
    if (len_path > 0 && buf[len_path - 1] != '/') {
        strcat(buf, "/");
    }

    // 追加文件名到路径
    strcat(buf, filename);
}



void set_permissions_recursive(char *path) {
    int fd;
    struct stat st;
    char buf[512];
    struct stat de;

    // 获取当前路径的信息
    if (stat(path, &st) < 0) {
        printf("init: cannot stat %s\n", path);
        return;
    }
    char mycwd[128];
    getcwd(mycwd);
    // 如果是目录，递归处理
    if (st.type == T_DIR) {
        if ((fd = open(path, 0)) < 0) {
            printf("init: cannot open directory %s\n", path);
            return;
        }

        // 遍历目录中的每个文件
        while (readdir(fd, &de) == 1) {
            if (de.name[0] == 0)  // 跳过无效目录项
                continue;
            printf("file %s chmod successfully!\n",de.name);
            // 跳过 "." 和 ".." 特殊目录
            if (strcmp(de.name, ".") == 0 || strcmp(de.name, "..") == 0)
                continue;
            
            // 构造完整路径
            build_path(buf, path, de.name, sizeof(buf));
            // 递归处理子目录或设置文件权限
            set_permissions_recursive(buf);
        }
        printf("close fd\n");
        close(fd);

        // 为目录设置权限
        chmod(path, 0755); // 目录权限: rwxr-xr-x
    } else {
        // 为普通文件设置权限
        chmod(path, 0644); // 文件权限: rw-r--r--
    }
}

int
main(void)
{
  int pid, wpid;

  // if(open("console", O_RDWR) < 0){
  //   mknod("console", CONSOLE, 0);
  //   open("console", O_RDWR);
  // }
  dev(O_RDWR, CONSOLE, 0);
  dup(0);  // stdout
  dup(0);  // stderr

  //初始化设置文件权限
  //set_permissions_recursive("/"); //fat32文件系统不支持文件权限，想要实现只能通过在内存建立映射表来模拟
  for(;;){
    printf("init: starting sh\n");
    pid = fork();
    if(pid < 0){
      printf("init: fork failed\n");
      exit(1);
    }
    if(pid == 0){
      exec("sh", argv);
      printf("init: exec sh failed\n");
      exit(1);
    }

    for(;;){
      // this call to wait() returns if the shell exits,
      // or if a parentless process exits.
      wpid = wait((int *) 0);
      if(wpid == pid){
        // the shell exited; restart it.
        break;
      } else if(wpid < 0){
        printf("init: wait returned an error\n");
        exit(1);
      } else {
        // it was a parentless process; do nothing.
      }
    }
  }
}
