#include "kernel/include/types.h"
#include "kernel/include/stat.h"
#include "xv6-user/user.h"

char*
fmtname(char *name)
{
  static char buf[STAT_MAX_NAME+1];
  int len = strlen(name);

  // Return blank-padded name.
  if(len >= STAT_MAX_NAME)
    return name;
  memmove(buf, name, len);
  memset(buf + len, ' ', STAT_MAX_NAME - len);
  buf[STAT_MAX_NAME] = '\0';
  return buf;
}

// 权限格式化
void
fmtperm(short mode, char *buf)
{
  // 文件类型
  buf[0] = (mode & 0400) ? 'r' : '-';
  buf[1] = (mode & 0200) ? 'w' : '-';
  buf[2] = (mode & 0100) ? 'x' : '-';
  buf[3] = (mode & 0040) ? 'r' : '-';
  buf[4] = (mode & 0020) ? 'w' : '-';
  buf[5] = (mode & 0010) ? 'x' : '-';
  buf[6] = (mode & 0004) ? 'r' : '-';
  buf[7] = (mode & 0002) ? 'w' : '-';
  buf[8] = (mode & 0001) ? 'x' : '-';
  buf[9] = '\0';
}

void
ls(char *path,int show_perm)
{
  int fd;
  struct stat st;
  char *types[] = {
    [T_DIR]   "DIR ",
    [T_FILE]  "FILE",
  };

  char perm_buf[10]; // 用于存储权限信息

  if((fd = open(path, 0)) < 0){
    fprintf(2, "ls: cannot open %s\n", path);
    return;
  }

  if(fstat(fd, &st) < 0){
    fprintf(2, "ls: cannot stat %s\n", path);
    close(fd);
    return;
  }

  char type;
  if (st.type == T_DIR){
    while(readdir(fd, &st) == 1){
      if (st.name == NULL) continue; // 跳过无效目录项
      //printf("%s %s\t%d\n", fmtname(st.name), types[st.type], st.size);
      if (show_perm) { // 显示权限信息
        if (st.mode >= 0) fmtperm(st.mode, perm_buf);
        if(st.type == T_DIR){
          type = 'd';
        }else{
          type = '-';
        }
        printf("%c%s %s %s\t%d\n",type, perm_buf, fmtname(st.name), types[st.type], st.size);
      } else { // 不显示权限信息
        printf("%s %s\t%d\n", fmtname(st.name), types[st.type], st.size);
      }

    }
  } else {
    //printf("%s %s\t%l\n", fmtname(st.name), types[st.type], st.size);
    if (show_perm) {
      if (st.mode >= 0) fmtperm(st.mode, perm_buf);
      printf("-%s %s %s\t%d\n", perm_buf, fmtname(st.name), types[st.type], st.size);
    } else {
      printf("%s %s\t%d\n", fmtname(st.name), types[st.type], st.size);
    }
    
  }
  close(fd);
}

int
main(int argc, char *argv[])
{
  int i;
  int show_perm = 0; // 默认不显示权限

  // 解析命令行参数
  for (i = 1; i < argc; i++) {
    if (strcmp(argv[i], "-a") == 0) {
      show_perm = 1;
      break; // 找到 -a 选项后退出循环
    }
  }

  if(argc < 2 || (argc == 2 && show_perm)){
    ls(".", show_perm);
    exit(0);
  }
  for(i=1; i<argc; i++){
    if (strcmp(argv[i], "-a") == 0) // 跳过 -a 选项
      continue;
    ls(argv[i], show_perm);
  }
  exit(0);
}
